# coding: utf-8

import RPi.GPIO as gpio
from time import sleep



# Poniższa numeracja pinów zakłada następujące połączenia kablowe:
#
# |   PIN MALINY  |  LEWY STEROWNIK SILNIKA | PRAWY STEROWNIK SILNIKA |
#  --------------- ------------------------- -------------------------
# |        9      |                         |          GND            |
# |       11      |                         |          SLP            |
# |       13      |                         |          STEP           | 
# |       15      |                         |          DIR            | 
# |               |                         |                         | 
# |       16      |            DIR          |                         | 
# |       18      |            STEP         |                         | 
# |       20      |            GND          |                         | 
# |       22      |            SLP          |                         | 
# 
# Oczywiście jeśli wybrało się inne wyjścia numery poniżej można 
# (i trzeba) pozmieniać
PIN_SILNIK_LEWY_KIERUNEK = 16    
PIN_SILNIK_LEWY_KROK = 18       
PIN_SILNIK_LEWY_WLACZ = 22

PIN_SILNIK_PRAWY_WLACZ = 11
PIN_SILNIK_PRAWY_KROK = 13
PIN_SILNIK_PRAWY_KIERUNEK = 15


# Stałe pozwalające czytelniej pisać kod. 
# Zamiast krok(-1, 1) można napisać krok(TYL, PRZOD).
# Obie instrukcje zrobią to samo, ale druga jest czytelniejsza.
PRZOD = 1
STOP = 0
TYL = -1

# Domyślny czas wykonywania kroku (5 milisekund). 
#
# Przy 5 milisekundach można wykonać maksymalnie 200 kroków/s, 
# co przy 60mm kole daje prędkość ~200mm/s. 
#
# Ustawienie większej wartości daje silnikowi więcej czasu 
# na wykonanie ruchu i jest on pewniejszy (silnik zawsze zdąży).
#
# Krótszy czas zwiększa maksymalną prędkość obrotową, ale może 
# powodować, że pierwsza lepsza przeszkoda obciąży silnik na tyle, 
# że nie zdąży on wykonać kroku i zablokuje się w miejscu.
#
# Poniżej 1ms silnik może się blokować nawet bez obciążenia.
DOMYSLNY_CZAS_KROKU = 0.005


def przygotuj():
    '''
    Ta funkcja przygotowuje piny Maliny do komunikacji ze sterownikami 
    silników. Należy ją wywołać na początku programu, przed wszystkimi 
    innymi. 
    '''
    gpio.setmode(gpio.BOARD)
    piny_sterowania_silnikow = [PIN_SILNIK_LEWY_KROK, 
        PIN_SILNIK_LEWY_KIERUNEK, PIN_SILNIK_LEWY_WLACZ, 
        PIN_SILNIK_PRAWY_KROK, PIN_SILNIK_PRAWY_KIERUNEK, 
        PIN_SILNIK_PRAWY_WLACZ]

    for pin in piny_sterowania_silnikow:    
        gpio.setup(pin, gpio.OUT)


def podlacz(lewy = 0, prawy = 0):
    '''
    Włącza lub wyłącza zasilanie silników. Na przykład:
    podlacz(1,0) -- włączy zasilanie w silniku lewym i wyłączy w prawym
    podlacz(0,0) -- wyłączy zasilanie obu silników
    podlacz(1, None) -- włącza zasilanie lewego silnika nie zmieniając 
                        stanu zasilania prawego
    podlacz() -- brak parametrów spowoduje przyjęcie domyślnych 0,0
                 a więc ta instrukcja wyłącza zasilanie obu silników
    '''
    # Przełączamy zasilanie silnika o ile wartość nie jest 
    # nieokreślona (None)
    if lewy is not None:
        gpio.output(PIN_SILNIK_LEWY_WLACZ, lewy)
    if prawy is not None:
        gpio.output(PIN_SILNIK_PRAWY_WLACZ, prawy)


def krok(lewy, prawy, czas = None):
    '''
    Wywołanie tej funkcji wysyła impusly sterujące do sterowników silnika, 
    które powodują wykonanie jednego kroku. 
    
    Na przykład:
       krok(PRZOD, PRZOD) -- wykonanie kroku w przód przez oba silniki
       krok(PRZOD, TYL)   -- lewy silnik wykona krok w przód, a prawy w tyl, 
                             pojazd skręci lekko w prawo w miejscu 
       krok(PRZOD, STOP)  -- lewy silnik wykona krok w przód, a prawy 
                             nie poruszy się

    Jeśli nie chcemy używać domyślnego czasu wykonywania kroku (patrz opis 
    zmiennej DOMYSLNY_CZAS_KROKU powyżej) możemy podać inną wartość czasu
    dla aktualnego kroku jako trzeci paramter. 
    
    Na przykład:
       krok(PRZOD, PRZOD, 0.01) -- krok do przodu na obu silnikach z względnie
                                   długim czasem -- w tym trybie powinno się 
                                   dać pokonać trudne przeszkody, które blokują
                                   silnik kiedy czas jest krótszy
    '''

    # Zamieniamy podane polecenie na odpowiednie wartości dla pinów STEP i DIR.

    if lewy == PRZOD:      # PRZOD: generujemy impuls kroku z pinem kierunku na 1
        lewy_krok = 1
        lewy_kierunek = 1    
    elif lewy == STOP:     # STOP: bez impulsu kroku, kierunek bez znaczenia
        lewy_krok = 0
        lewy_kierunek = None
    elif lewy == TYL:      # TYL: impuls kroku z pinem kierunku na 0  
        lewy_krok = 1
        lewy_kierunek = 0
    else:
        raise "Nieznane polecenie!"  # Nie PRZOD, nie TYL, nie STOP??? To co?   

    # Jak wyżej, tylko dla prawego silnika. Zwróć uwagę, że pin kierunku włączany
    # jest odwrotnie (dla PRZOD ma wartosc 1, dla TYL 0). Wynika to z budowy 
    # robota gdzie silniki są obrócone do siebie o 180 stopni i trzeba je obracać 
    # w przeciwną stronę, żeby napędzały robota w tym samym kierunku.
    if prawy == PRZOD:       
        prawy_krok = 1
        prawy_kierunek = 0
    elif prawy == STOP:  
        prawy_krok = 0
        prawy_kierunek = None
    elif prawy == TYL:   
        prawy_krok = 1
        prawy_kierunek = 1
    else:
        raise "Nieznane polecenie!"  

    # Jeśli czas nie jest podany ustawiamy domyślną wartość zdefiniowaną
    # w DOMYSLNY_CZAS_KROKU
    if czas is None:
        czas = DOMYSLNY_CZAS_KROKU

    # I wykonanie kroku:
 
    # Najpierw ustawiamy sygnały na pinach kierunków o ile jest to potrzebne 
    # (np. polecenie STOP nie wymaga ustawiania kierunku)
    if lewy_kierunek is not None:
        gpio.output(PIN_SILNIK_LEWY_KIERUNEK, lewy_kierunek)

    if prawy_kierunek is not None:
        gpio.output(PIN_SILNIK_PRAWY_KIERUNEK, prawy_kierunek)

    # Teraz wysyłamy impuls wykonania kroku, czyli przez połowę podanego czasu 
    # ustawiamy pin na wyznaczoną powyżej wartość, a po chwili dajemy 0. 
    gpio.output(PIN_SILNIK_LEWY_KROK, lewy_krok)
    gpio.output(PIN_SILNIK_PRAWY_KROK, prawy_krok)
    sleep(czas*0.5)

    gpio.output(PIN_SILNIK_LEWY_KROK, 0)
    gpio.output(PIN_SILNIK_PRAWY_KROK, 0)
    sleep(czas*0.5)

    # Dokładniejszy opis: 
    #
    # Jeśli wyznaczona wartość ..._krok wynosiła 1, sygnał na pinie STEP skoczy 
    # do 1 (czyli ok. 3 wolty) i potem spadnie do 0. Sterownik zinterpretuje 
    # to jako impuls na wejściu STEP i wygeneruje odpowiednie sygnały 
    # na wyjściach silnika, które przekręcą go o krok. 
    #
    # Jeśli wartość ..._step wynosi 0, wówczas sygnał na pinie będzie cały czas 
    # ustawiony na 0, sterownik nie otrzyma impulsu na wejściu STEP i nie 
    # wygeneruje nowych sygnałów na wyjściu silnika, który wobec tego pozostanie 
    # w niezmienionej pozycji.
    #
    # Wykresy napięcia jakie "widzi" wejście STEP lewego sterownika zależnie 
    # od wartości lewy_krok:
    # 
    #
    #                   |         1        |                  |
    # 3V                |   ...............|                  |
    #                   |  .               |                  | 
    #  lewy_krok = 1    | .                |.                 |
    #                   |.                 | .      0         |
    # 0V ...............|                  |  ................|.............
    #                   |                  |                  |
    #                   |                  |                  |
    #                   |                  |                  |
    # 3V                |                  |                  |
    #                   |                  |                  | 
    #  lewy_krok = 0    |                  |                  |
    #                   |         0        |        0         |
    # 0V ...............|..................|..................|.............
    #                   |                  |                  |
    #                     <-- czas*0.5 -->   <-- czas*0.5 -->     
    #
    #                     <------------  czas  ------------->


def zestaw(lewo, prawo, powtorek = 1):
    '''
    Funkcja bierze polecenie i powtarza jego wykonanie podaną ilość razy: 
    lewo - polecenie dla lewego silnika
    prawo - polecenie dla prawego silnika
    powtorek - oznacza ile razy nalezy ten zestaw kroków powtórzyć

    Na przykład:
    zestaw(PRZOD, STOP, 10)   -- wykona 10 kroków do przodu lewym silnikiem,
    zestaw(PRZOD, PRZOD, 200) -- wykona 200 kroków do przodu oboma silnikami,
                                 a więc koła obrócą się o pełny obrót  
    zestaw(PRZOD, TYL, 100)   -- obróci lewe koło 100 kroków do przodu, 
                                 a prawe 100 do tyłu, a więc robot powinien 
                                 wykonać skręt w prawo w miejscu.
    ''' 
    for i in xrange(1, 1 + powtorek):
        krok(lewo, prawo)
    

def sekwencja(lista_zestawow):
    '''
    Czyta podaną listę i wykonuje kolejno podane w niej zestawy kroków.

    lista_zestawów -- lista zawierająca kolejne zestawy; w pythonie 
                      listę podaje się w nawiasach kwadratowych [] 
                      oddzielając elementy przecinkami 

    Na przykład:

    sekwencja([ (PRZOD, PRZOD, 400), (PRZOD, TYL, 100), (TYL, PRZOD, 100) ]) 

         -- wykona kolejno trzy zestawy: 
            1. (PRZOD, PRZOD, 400)
            2. (PRZOD, TYL, 100)
            3. (TYL, PRZOD, 100)
            Pierwszy spowoduje podjechanie modelu o dwa pełne obroty kół 
            do przodu. Drugi obrócenie lewego koła do przodu a prawego 
            do tyłu o pół obrotu, a wiec robot skręci w miejscu w prawo. 
            Ostatni zestaw spowoduje skręt w miejscu w przeciwną stronę 
            - robot powróci do pierwotnego kierunku. Cała sekwencja wyglądać 
            będzie tak jakby robot podjechał do przodu i rozejrzał się 
            w prawo. 
    '''
    for kolejny_zestaw in lista_zestawow:
        zestaw(*kolejny_zestaw)

